library open_library;

import 'dart:typed_data';

import 'service/api_service.dart';
import 'models/private/api_author_model.dart';
import 'models/private/api_book_model.dart';
import 'models/ol_author_model.dart';
import 'models/ol_book_model.dart';

/// OpenLibrary class to support an easy way to consume openlibrary.org api
/// Version 0.0.1
/// Author: Andreas Lange
/// Published in 2022
class OpenLibrary {
  final OpenLibraryApiService _openLibraryApiService =
      OpenLibraryApiService.create();

  void dispose() {
    _openLibraryApiService.client.dispose();
  }

  /// Get a book by it's ISBN number (10 or 13 digits) from OpenLibrary.org
  /// Always check RuntimeType OLBookBase type is -> OLBook / OLBookError
  Future<OLBookBase> getBookByISBN({required String isbn}) async {
    // try to fetch the book via isbn from the openlibrary.org isbn api
    final APIBookBase book =
        await _openLibraryApiService.getBookByISBN(isbn: isbn);
    if (book is APIBook) {
      // fetch Authors if exist
      final List<OLAuthor> authors = [];
      if (book.authors.isNotEmpty) {
        // each author needs to be fetched from openlibrary.org
        await Future.forEach(book.authors, (Map<String, String> element) async {
          if (element.keys.contains('key')) {
            final String id = element['key'] ?? '';
            if (id.isNotEmpty) {
              // the api wraps the id so we can remove path elements here
              final APIAuthorBase author = await _openLibraryApiService
                  .getAuthorById(id: id.replaceFirst('/authors/', ''));
              if (author is APIAuthor) {
                authors.add(OLAuthor.fromJson(author.toJson()));
              }
            }
          }
        });
      }

      // fetch covers if exists
      final List<Uint8List> covers = [];
      if (book.covers.isNotEmpty) {
        // each cover needs to be downloaded from the openlibrary.org covers endpoint
        await Future.forEach(book.covers, (coverId) async {
          // we will provide byte data as Uint8List so it can be used directly as MemoryImage
          final Uint8List cover =
              await _openLibraryApiService.getCoverById(id: coverId.toString());
          if (cover.isNotEmpty) {
            covers.add(cover);
          }
        });
      }

      // creates a new OLBook instance to be returned as data copy of
      // it's api transfer object APIBook with added authors and covers
      final OLBook result = OLBook(
        title: book.title,
        full_title: book.full_title,
        subtitle: book.subtitle,
        authors: authors,
        contributions: book.contributions,
        edition_name: book.edition_name,
        ia_box_id: book.ia_box_id,
        isbn_10: book.isbn_10,
        isbn_13: book.isbn_13,
        latest_revision: book.latest_revision,
        revision: book.revision,
        number_of_pages: book.number_of_pages,
        ocaid: book.ocaid,
        lc_classifications: book.lc_classifications,
        pagination: book.pagination,
        physical_dimensions: book.physical_dimensions,
        physical_format: book.physical_format,
        publish_date: book.publish_date,
        publish_places: book.publish_places,
        publishers: book.publishers,
        subjects: book.subjects,
        weight: book.weight,
        covers: covers,
      );
      return result;
    }
    // in the case of an error return the Error Instance
    return const OLBookError();
  }

  /// Get an auther by his's OL ID number from OpenLibrary.org
  /// Always check RuntimeType OLAuthorsBase type is -> OLBAuthor / OLAuthorError
  Future<OLAuthorBase> getAuthorByOLID({required String olid}) async {
    final APIAuthorBase author =
        await _openLibraryApiService.getAuthorById(id: olid);
    if (author is APIAuthor) {
      final OLAuthor result = OLAuthor.fromJson(author.toJson());
      return result;
    }
    return const OLAuthorError();
  }
}
