import 'dart:convert';
//import 'dart:developer';
import 'dart:typed_data';

import 'package:chopper/chopper.dart';
import '../models/private/api_author_model.dart';
import '../models/private/api_book_model.dart';

part 'api_service.chopper.dart';

@ChopperApi(baseUrl: '')
abstract class OpenLibraryApiService extends ChopperService {
  static OpenLibraryApiService create() {
    final client = ChopperClient(
      baseUrl: '',
      services: [
        _$OpenLibraryApiService(),
      ],
      converter: const ApiConverter(type: APIBook),
    );

    return _$OpenLibraryApiService(client);
  }

  Future<APIBookBase> getBookByISBN({required String isbn}) async {
    final Response<APIBook> book = await _getBookByISBN(isbn: isbn);
    if (book.statusCode == 200 && book.body != null) {
      return book.body!;
    }
    return const APIBookError();
  }

  Future<APIAuthorBase> getAuthorById({required String id}) async {
    final Response<APIAuthor> author = await _getAuthorById(id: id);
    if (author.statusCode == 200 && author.body != null) {
      return author.body!;
    }
    return const APIAuthorError();
  }

  Future<Uint8List> getCoverById({required String id}) async {
    final Response<Uint8List> cover = await _getCoverById(id: id);
    if (cover.statusCode == 200 && cover.body != null) {
      return cover.body!;
    }
    return Uint8List(0);
  }

  @Get(path: 'https://openlibrary.org/isbn/{isbn}.json')
  Future<Response<APIBook>> _getBookByISBN(
      {@Path('isbn') required String isbn});

  @Get(path: 'https://openlibrary.org/authors/{id}.json')
  Future<Response<APIAuthor>> _getAuthorById({@Path('id') required String id});

  @Get(path: 'https://covers.openlibrary.org/b/id/{id}-L.jpg')
  Future<Response<Uint8List>> _getCoverById({@Path('id') required String id});
}

class ApiConverter extends JsonConverter {
  final dynamic type;
  const ApiConverter({required this.type});

  @override
  Response convertError<BodyType, InnerType>(Response response) {
    return response.copyWith(body: null);
  }

  @override
  Response decodeJson<BodyType, InnerType>(Response response) {
    return response;
  }

  @override
  Request encodeJson(Request request) {
    return request;
  }

  @override
  Request convertRequest(Request request) {
    return super.convertRequest(request.copyWith(body: null));
  }

  @override
  Response<BodyType> convertResponse<BodyType, InnerType>(Response response) {
    final Response dynamicResponse = super.convertResponse(response);
    //log("BodyType:$BodyType -> ${BodyType == Book}");
    if (BodyType == APIBook) {
      final BodyType customBody = _convertBook(dynamicResponse.body);
      return dynamicResponse.copyWith<BodyType>(body: customBody);
    } else if (BodyType == APIAuthor) {
      final BodyType customBody = _convertAuthor(dynamicResponse.body);
      return dynamicResponse.copyWith<BodyType>(body: customBody);
    } else if (BodyType == Uint8List) {
      final BodyType customBody = dynamicResponse.bodyBytes as BodyType;
      return dynamicResponse.copyWith<BodyType>(body: customBody);
    }
    return dynamicResponse.copyWith<BodyType>(body: null);
  }

  dynamic _convertBook(dynamic element) {
    //log("content<$APIBook>:$element",name:"API_CONVERTER");
    return APIBook.fromJson(jsonDecode(element));
  }

  dynamic _convertAuthor(dynamic element) {
    //log("content<$APIAuthor>:$element",name:"API_CONVERTER");
    return APIAuthor.fromJson(jsonDecode(element));
  }
}
